package UI.GameState;

import UI.LogIn.MainPanel;

import javax.imageio.ImageIO;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Random;

/**
 * represents the walls of the game
 * it has a subclass that contains each class data
 * @author zahra_fatehi
 * @version 0.0
 */
public class WallState {
    private int frameX, frameY;

    private String source = "C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\maps\\map3.txt";
    private String[] lines;
    public int numRows, numColumns, w, h;
    private ArrayList<WallData> walls = new ArrayList<>();
    private Image wall1 ;
    private Image wall2 ;
    private Image blank ;

    /**
     * create the walls for the game
     * @param GAME_WIDTH is the width of the frame
     * @param GAME_HEIGHT is the height of the frame
     */
    public WallState(int GAME_WIDTH, int GAME_HEIGHT) {
        frameX = GAME_WIDTH;
        frameY = GAME_HEIGHT;

        try {
            if (MainPanel.singlePlay)
            setSource(findMapPath());
            wall1 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall1.png"));
            wall2 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall2.png"));
            blank = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\blank.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        lines = source.split("\r\n|\r|\n");

        numRows = lines.length;  //number of rows
        numColumns = lines[0].trim().length();  //number of columns

        h = (frameY-30) / (numRows);   //sideY
        w = (frameX-10) / (numColumns);  //sideX

        wall1 = wall1.getScaledInstance(w,h,Image.SCALE_SMOOTH);
        wall2 = wall2.getScaledInstance(w,h,Image.SCALE_SMOOTH);
        blank = blank.getScaledInstance(w,h,Image.SCALE_SMOOTH);

        loadWalls();
    }

    /**
     * thois method is for drawing components
     * @param g is the graphic which we use it
     */
    public void paintComponent(Graphics g) {

        g.setColor(Color.WHITE);
        g.fillRect(0, 0, frameX, frameY);
        g.setColor(Color.black);  //colour of the walls

        for (WallData d :walls){
            drawWall(g,d);
        }
    }

    /**
     * this method is for drawing the walls
     */
    public void loadWalls() {
        // Draw background
        walls.clear();
        int x = 0;
        int y = 0;

        for (int r = 0; r < numRows; r++) {
            x = 0 ;
            String rowChars = lines[r];
            for (int c = 0; c < numColumns; c++) {
                if (rowChars.charAt(c) == '0') {
                    saveBlank( x, y ,3);
                    x += w;
                }
                else if (rowChars.charAt(c) == '1') {
                    saveWalls(x, y, r , c, 1);
                    x += w;
                }
                else if (rowChars.charAt(c) == '2') {
                    saveWalls( x, y, r , c, 2);
                    x += w;
                }
            }
            y += h;
        }
    }


    /**
     *this method is to save walls
     * @param x of the wall
     * @param y of the wall
     * @param r of the wall
     * @param c of the wall
     * @param image is the image of wall
     */
    private void saveWalls(int x,int y , int r , int c ,int image){
        WallData wall = new WallData(x , y ,image,false);
        if ((c<numColumns-1 && lines[r].charAt(c+1)=='1') || (c>0 && lines[r].charAt(c-1)=='1')||(c<numColumns-1 && lines[r].charAt(c+1)=='2') || (c>0 && lines[r].charAt(c-1)=='2')){ //horizontal
            wall.setHorizontal(true);
        }
        if ((r>0 && lines[r-1].charAt(c)=='1') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='1')||(r>0 && lines[r-1].charAt(c)=='2') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='2')) { //vertical
            wall.setVertical(true);
        }
        if (lines[r].charAt(c)=='2'){
            wall.setDestructible(true);
        }
        walls.add(wall);
    }

    /**
     * this method is for drawing the walls
     * @param g2d is the graphic we use
     * @param data is the data if the wall
     */
    private void drawWall(Graphics g2d,WallData data){
        Image img;
        if (data.image==1) {
            img = wall1;
        }else if (data.image==2){
            img = wall2;
        }else{
            img = blank;
        }
        g2d.drawImage(img,data.x,data.y,null);
    }

    /**
     * this method is to  save blank places
     * @param x for wall
     * @param y for wall
     * @param image of the wall
     */
    private void saveBlank(int x,int y,int image){
        WallData wall = new WallData(x , y ,image,true);
        walls.add(wall);
    }
    /**
     * to know that bullet can get this position or not
     * @param x is the x of tank
     * @param y is the y of tank
     * @return the wall position
     */
    public WallData getWallForBullet(int x,int y){
        for(WallData d : walls){
            if (d.isWallForBullet(x,y))
                return d;
        }
        return null;
    }

    /**
     * to know that tank can get this position or not
     * @param x is the x of tank
     * @param y is the y of tank
     * @return true if tank can get the place or not
     */
    public boolean isWallForTank(int x,int y){
        for(WallData d : walls){
            if (d.isWallForTank(x,y))
                return true;
        }
        return false;
    }

    /**
     * choose a map randomly
     * @return
     */
    public String findMapPath(){
        Random random = new Random();
        int x = random.nextInt(2);
        System.out.println(x);
        switch (x){
            case 0:return "C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\maps\\map1.txt";
            case 1:return "C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\maps\\map3.txt";
        }
        return null;
    }

    /**
     * set the path of the map
     * @param mapPath
     * @throws IOException
     */
    public void setSource(String mapPath) throws IOException {
        source = new String(Files.readAllBytes(Paths.get(mapPath)), StandardCharsets.UTF_8);
    }

    public ArrayList<WallData> getWalls() {
        return walls;
    }

    /**
     * this class is a subclass foe wall state that saves the data of the wall
     */
    public class WallData {
        private int image;
        private int x,y;
        private boolean horizontal = false;
        private boolean vertical = false;
        private boolean blank;
        private boolean destructible = false;

        public WallData(int x,int y,int image,boolean isBlank){
            this.x = x;
            this.y = y;
            this.image = image;
            this.blank = isBlank;
        }

        /**
         *some getter and setters
         */
        public boolean isWallForTank (int x,int y){
            //BUG
            if (blank) return false;
            return (x>this.x-w/2 && x<this.x+w && y>this.y-h/2 && y<this.y+h);
        }

        public boolean isWallForBullet (int x,int y){
            //BUG
            if (blank) return  false;
            return (x>this.x-10 && x<this.x+w+10 && y>this.y-10 && y<this.y+h+10);
        }

        public boolean isHorizontal() {
            return horizontal;
        }

        public void setHorizontal(boolean horizontal) {
            this.horizontal = horizontal;
        }

        public boolean isVertical() {
            return vertical;
        }

        public void setVertical(boolean vertical) {
            this.vertical = vertical;
        }

        public boolean isDestructible() {
            return destructible;
        }

        public void setDestructible(boolean destructible) {
            this.destructible = destructible;
        }

        public int getX() {
            return x;
        }

        public int getY() {
            return y;
        }

    }
}
